import java.awt.*;

/**
 * A noninstantiable utility class to manipulate Color objects.
 * 
 * @author Tim Bahls
 * 
 */

public class ColorOps {

	// a precomputed constant for quicker calculations
	private static final double oneThirdRotation = Math.PI * 2 / 3;
	// a precomputed constant for quicker calculations
	private static final double twoThirdsRotation = Math.PI * 4 / 3;

	/**
	 * return a darker version of the given Color
	 * 
	 * @param c
	 *            the Color to darken
	 * @return the darker Color
	 */
	public static Color darken(Color c) {
		return new Color(c.getRed() / 2, c.getGreen() / 2, c.getBlue() / 2);
	}

	/**
	 * return a lighter version of the given Color
	 * 
	 * @param c
	 *            the Color to lighten
	 * @return the lighter Color
	 */
	public static Color lighten(Color c) {
		return new Color((c.getRed() + 255) / 2, (c.getGreen() + 255) / 2, (c
				.getBlue() + 255) / 2);
	}

	/**
	 * Linearly interpolates between two colors based on n. If n is 0, this
	 * returns a Color equal to c2. If n is 1, you get c1 instead. A c value of
	 * .5 will return the average, and so forth.
	 * 
	 * @param c1
	 *            the first Color
	 * @param c2
	 *            the second Color
	 * @param n
	 *            the ratio of c2 to use
	 * @return the interpolated color.
	 */
	public static Color interpolate(Color c1, Color c2, float n) {
		float m = 1 - n;
		return new Color((int) (n * c1.getRed() + m * c2.getRed()), (int) (n
				* c1.getGreen() + m * c2.getGreen()),
				(int) (n * c1.getBlue() + m * c2.getBlue()));
	}

	/**
	 * Return a bright, saturated Color with a random hue
	 * 
	 * @return a bright, saturated Color
	 */
	public static Color getBrightColor() {
		double d = Math.random() * 2 * Math.PI;
		return getBrightColor(d);
	}

	/**
	 * Return a bright, saturated Color with a hue based on d
	 * 
	 * @param d
	 *            the location along the rainbow to choose
	 * @return the specified bright, saturated Color
	 */
	public static Color getBrightColor(double d) {
		int redmap = (int) (128 * Math.sin(d) + 128);
		int greenmap = (int) (128 * Math.sin(d + oneThirdRotation) + 128);
		int bluemap = (int) (128 * Math.sin(d + twoThirdsRotation) + 128);
		return new Color(redmap, greenmap, bluemap);
	}

}
