import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Polygon;

/**
 * A Laser that can move and draw itself
 * 
 * @author Jim Skrentny modified
 * @author Tim Bahls
 * 
 */
public class Laser {

	// the angle the diagonal makes with the direction of travel
	private static final double DIAG_ANGLE = .2;
	// the ditance from the center to the corners
	private static final double SHOT_RADIUS = 15;
	// The color of the laser
	private Color color;
	// the current x position
	private int xPosition;
	// the current y position
	private int yPosition;
	// the current x velocity
	private int yVelocity;
	// the current y velocity
	private int xVelocity;
	// true if this Laser was fired by the Defender
	private boolean defenderSide;

	/**
	 * Constructs a Laser based on the given location and velocity
	 * 
	 * @param startXPosition
	 *            the starting x position
	 * @param startYPosition
	 *            the starting y position
	 * @param xVelocity
	 *            the starting x velocity
	 * @param yVelocity
	 *            the starting y velocity
	 */
	public Laser(int startXPosition, int startYPosition, int xVelocity,
			int yVelocity) {
		this.xPosition = startXPosition;
		this.yPosition = startYPosition;
		this.yVelocity = yVelocity;
		this.xVelocity = xVelocity;
		color = ColorOps.getBrightColor();
		defenderSide = false;
	}

	/**
	 * Constructs a new Laser based on the position of the Defender
	 * 
	 * @param d
	 *            the Defender
	 */
	public Laser(Defender d) {
		this.xPosition = d.getXPosition();
		this.yPosition = d.getYPosition();
		//JIM changed 2 lines below
		this.yVelocity = SpaceGameMain.rng.nextInt(3) - 11; //range -11 to -9 inclusive
		this.xVelocity = SpaceGameMain.rng.nextInt(3) - 1; //range -1 to 1 inclusive
		color = ColorOps.getBrightColor();
		defenderSide = true;
	}

	/**
	 * Returns true if the Defender fired this Laser
	 * 
	 * @return true if the Defender fired this Laser
	 */
	public boolean isDefenderLaser() {
		return defenderSide;
	}

	/**
	 * Updates the position of the Laser for the next time it is redrawn.
	 */
	public void takeOneStep() {
		yPosition += yVelocity;
		xPosition += xVelocity;
	}

	/**
	 * Returns true if this Laser has left the game area.
	 * 
	 * @return true if this Laser has left the game area.
	 */
	public boolean removeMeFromGame() {
		if (xPosition < 0 || yPosition < 0) {
			return true;
		}
		if (xPosition > SpaceGame.getMaximumX()
				|| yPosition > SpaceGame.getMaximumY()) {
			return true;
		}
		return false;
	}

	/**
	 * Paint this Laser onto the given Graphics object
	 * 
	 * @param g
	 *            the Graphics object to draw onto
	 */
	public void paint(Graphics g) {
		// draw a rectangle rotated to line up with the direction of travel

		// first find the angle
		double angle = Math.atan2(yVelocity, xVelocity);

		// next find the four corners
		Point p1 = polar(xPosition, yPosition, angle + DIAG_ANGLE, SHOT_RADIUS);
		Point p2 = polar(xPosition, yPosition, angle - DIAG_ANGLE, SHOT_RADIUS);
		Point p3 = polar(xPosition, yPosition, angle + Math.PI + DIAG_ANGLE,
				SHOT_RADIUS);
		Point p4 = polar(xPosition, yPosition, angle + Math.PI - DIAG_ANGLE,
				SHOT_RADIUS);

		// Store the corners in a Polygon
		Polygon poly = new Polygon();
		poly.addPoint(p1.x, p1.y);
		poly.addPoint(p2.x, p2.y);
		poly.addPoint(p3.x, p3.y);
		poly.addPoint(p4.x, p4.y);

		// Draw the polygon
		g.setColor(color);
		g.fillPolygon(poly);

	}

	// geometric helper method
	private static Point polar(int origX, int origY, double angle, double radius) {
		int x = origX + (int) (Math.cos(angle) * radius);
		int y = origY + (int) (Math.sin(angle) * radius);
		return new Point(x, y);
	}

	/**
	 * Get the current x coordinate of the center of this Laser.
	 * 
	 * @return The current x coordinate of the center of this Laser.
	 */
	public int getXPosition() {
		return this.xPosition;
	}

	/**
	 * Get the current y coordinate of the center of this Laser.
	 * 
	 * @return The current y coordinate of the center of this Laser.
	 */
	public int getYPosition() {
		return this.yPosition;
	}

}
