import java.awt.*;
import java.util.Vector;

/**
 * The class that arranges all the pieces into a coherent game. Also represents
 * the window itself that appears on the screen.
 * 
 * @author Jim Skrentny modified
 * @author Tim Bahls
 * 
 */
public class SpaceGame extends GameFrame implements ImageGetter {

	/**
	 * The height and width of the UFO objects
	 */
	public static final int UFO_SIZE = 80;
	/**
	 * The height and width of the Defender object
	 */
	public static final int DEFENDER_SIZE = 80;

	// A reference to the game itself. It lets us get the height and width of the
	// game without needed a reference in that particular context
	private static SpaceGame theGame;

	// the list of UFOs
	private Vector<UFO> uFOs;
	// the list of shots
	private Vector<Laser> shots;
	// the defender object
	private Defender defender;
	// the list of filenames
	private String[] uFOPics = { "smallguy", "saucer", "speeder" };

	/**
	 * Contructs a SpaceGame object that will have have the specified dimensions
	 * 
	 * @param width
	 *            the width
	 * @param height
	 *            the height
	 */
	public SpaceGame(int width, int height) {
		super("Defend planet Earth", width, height);
		uFOs = new Vector<UFO>();
		shots = new Vector<Laser>();
		clearColor = Color.BLACK;
		maxFPS = 30;
		ImageCache.setImageGetter(this);
		ImageCache.setExtension(".gif");
		theGame = this;
		addMouse();
		defender = new Defender(hrz / 2, vrt - DEFENDER_SIZE / 2, "defender");
	}

	/**
	 * Gets the x coordinate of the rightmost column of pixels
	 * 
	 * @return the x coordinate
	 */
	public static int getMaximumY() {
		return theGame.vrt - 1;
	}

	/**
	 * Gets the y coordinate of the bottommost row of pixels
	 * 
	 * @return the y coordinate
	 */
	public static int getMaximumX() {
		return theGame.hrz - 1;
	}

	/**
	 * Update all game objects
	 */
	public void tick() {
		// update all UFOs
		for (int i = 0; i < uFOs.size(); i++) {
			UFO e = uFOs.get(i);
			e.takeOneStep(defender);

			if (e.removeMeFromGame()) {
				// remove dead UFOs
				uFOs.remove(i);
				i--;
			} else {
				// all UFOs that wish to fire
				if (e.shootsThisTurn(defender)) {
					shots.add(e.fireWeapon(defender));
				}
			}
		}
		// update all shots
		for (int i = 0; i < shots.size(); i++) {
			Laser b = shots.get(i);
			b.takeOneStep();
			boolean removeMe = false;
			if (!b.isDefenderLaser()) {
				// all UFO shots might hit the Defender
				if (defender != null && defender.hitByLaser(b)) {
					defender.wasHit();
					removeMe = true;
				}
			} else {
				// all Defender shots might hit the UFOs
				for (int j = 0; j < uFOs.size(); j++) {
					UFO e = uFOs.get(j);
					if (e.isHitByLaser(b)) {
						e.recordHit();
						removeMe = true;
					}
				}
			}
			// remove all dead shots
			if (removeMe || b.removeMeFromGame()) {
				shots.remove(i);
				i--;
			}
		}
		// kill the defender if necesary
		if (defender != null) {
			if (defender.removeMeFromGame()) {
				defender = null;
			}
		}
	}

	/**
	 * Add a UFO with the specified type to the SpaceGame.
	 * @param type the type of UFO to add
	 */
	public void addUFO(int type) {
		if (type >= 0 && type < uFOPics.length) {
			//JIM changed 3 lines below
			int hsize = UFO_SIZE / 2;
			int x = SpaceGameMain.rng.nextInt(hrz) + hsize;
			int y = SpaceGameMain.rng.nextInt(vrt-2*UFO_SIZE) + hsize;
			uFOs.add(new UFO(x, y, type));
		} else {
			throw new RuntimeException("No such enemy type " + type
					+ ".  Valid types are from 0 to " + (uFOPics.length - 1));
		}
	}

	/**
	 * Paint all game objects onto the given Graphics object
	 * @param g the Graphics object to draw onto
	 */
	public void paint(Graphics g) {
		if (uFOs == null || shots == null) {
			return;
		}
		//paint all UFO objects
		for (int i = 0; i < uFOs.size(); i++) {

			UFO ufo = uFOs.get(i);
			int type = ufo.getUFOType();
			if (type >= 0 && type < uFOPics.length) {
				Image image = ImageCache.getIm(uFOPics[type]);
				int x = ufo.getXPosition() - UFO_SIZE / 2;
				int y = ufo.getYPosition() - UFO_SIZE / 2;
				g.drawImage(image, x, y, null);
			} else {
				throw new RuntimeException("No such enemy type " + type);
			}

		}
		//paint all Lasers
		for (int i = 0; i < shots.size(); i++) {
			shots.get(i).paint(g);
		}
		//Paint the defender (if alive)
		if (defender != null) {
			defender.paint(g);

		}
		//Print game results (if any)
		String result = "";
		if (defender == null) {
			result = "YOU LOST, BUT THAT'S OKAY!";
		} else if (uFOs.size() == 0) {
			result = "YOU WON!  HOORAY!  YIPEE!";
		}
		g.setColor(Color.WHITE);
		g.setFont(new Font("SansSerif", 1, 30));
		g.drawString(result, hrz / 2 - 200, vrt / 2);
	}

	/**
	 * This is called any time the mouse is moved, pressed, released, or
	 * dragged.
	 * 
	 * @param mx the x cooridinate of the mouse action
	 * @param my the x cooridinate of the mouse action
	 * @param pressed true if a mouse button was just pressed
	 * @param released true if a mouse button was just released
	 * @param mouseButton the code for this particular mouse button
	 */
	public void mouse(int mx, int my, boolean pressed, boolean released,
			int mouseButton) {
		if (defender == null) {
			return;
		}
		defender.setXPosition(mx);
		if (pressed) {
			shots.add(defender.fireWeapon());
		}
	}
	
	/**
	 * Get the file with the given filename from the filesystem.
	 * @param fileName the filename
	 * @return the image
	 */
	public Image getIm(String fileName) {
		return Toolkit.getDefaultToolkit().getImage(fileName);
	}

	/**
	 * Display the SpaceGame window and start the game
	 */
	public void start(){
		super.start();
	}

}
